/**
 * \file
 * Inteface for common socket operations. (bind, send, recv)
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __TR_INTERFACE_SOCKET_H__
#define __TR_INTERFACE_SOCKET_H__

#include <sys/types.h>

#include "trbase.h"
#include "tr/remote_data.h"

typedef int           (* fptr_TR_socketBind)(void *);
typedef int           (* fptr_TR_socketConnect)(void *);
typedef TR_RemoteData (* fptr_TR_socketRecv)(void *, size_t);
typedef ssize_t       (* fptr_TR_socketSend)(void *, TR_RemoteData);

TR_INTERFACE(TR_Socket) {
	TR_IFID;
	fptr_TR_socketBind    bind;
	fptr_TR_socketConnect connect;
	fptr_TR_socketRecv    recv;
	fptr_TR_socketSend    send;
};

inline
int
TR_socketBindAction(void * _this)
{
	int callret;
	TR_RETCALL(_this, TR_Socket, bind, callret);
	return callret;
}

inline
int
TR_socketConnectAction(void * _this)
{
	int callret;
	TR_RETCALL(_this, TR_Socket, connect, callret);
	return callret;
}

extern TR_RemoteData TR_socketRecv(void *, size_t);
extern ssize_t       TR_socketSend(void *, TR_RemoteData);

#endif // __TR_INTERFACE_SOCKET_H__

// vim: set ts=4 sw=4:
