/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include "tr/remote_data.h"
#include "tr/socket.h"
#include "tr/interface/socket.h"
#include "tr/logger.h"
#include "trbase.h"

static
int
socketCtor(void * _this, va_list * params)
{
	TR_Socket   this  = _this;
	char      * host;

	this->type      = AF_UNSPEC;
	this->handle    = -1;
	this->log       = va_arg(*params, TR_Logger);
	host            = va_arg(*params, char*);
	this->port      = va_arg(*params, int);
	this->flags     = va_arg(*params, int);
	this->fin_state = TR_FIN_RDWR;

	if (host) {
		this->host[TR_MAX_HOST-1] = 0;
		strncpy(this->host, host, TR_MAX_HOST-1);
	}

	return 0;
}

static
void
socketDtor(void * _this)
{
	TR_Socket this = _this;

	if (STDERR_FILENO < this->handle) {
		TR_socketClose(this);
	}
}

static
int
socketBind(void * _this)
{
	TR_Socket this = _this;
	int       state;

	state = bind(this->handle, &(this->addr.info), this->addrlen);

	if (-1 == state) {
		perror("bind");
	}

	return state;
}

TR_INIT_IFACE(TR_Class, socketCtor, socketDtor, NULL);
TR_INIT_IFACE(TR_Socket, socketBind, NULL, NULL);
TR_CREATE_CLASS(TR_Socket, NULL, NULL, TR_IF(TR_Class), TR_IF(TR_Socket));

// vim: set ts=4 sw=4:
