/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>      // for errno
#include <unistd.h>
#include <fcntl.h>

#include "tr/socket.h"
#include "tr/logger.h"

int
TR_socketGetFd(TR_Socket this)
{
	struct msghdr    msg;
	struct cmsghdr * cmsg;
	char             cmsgbuf[CMSG_SPACE(sizeof(int))];
	int              fd = 0;

	memset(&msg, 0, sizeof(msg));
	msg.msg_control    = cmsgbuf;
	msg.msg_controllen = sizeof(cmsgbuf);

	if (0 > recvmsg(TR_socketHandle(this), &msg, 0)) {
		return -1;
	}

	cmsg = CMSG_FIRSTHDR(&msg);
	if (cmsg == NULL || cmsg->cmsg_type != SCM_RIGHTS) {
		return -1;
	}

	memcpy(&fd, CMSG_DATA(cmsg), sizeof(int));
	return fd;
}

// vim: set ts=4 sw=4:
