/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include "tr/socket.h"
#include "tr/interface/socket.h"
#include "tr/logger.h"
#include "trbase.h"

static
int
tcpSocketCtor(void * _this, va_list * params)
{
	TR_TcpSocket this = _this;

	TR_PARENTCALL(TR_TcpSocket, _this, TR_Class, ctor, params);
	TR_socketType((TR_Socket)this) = SOCK_STREAM;

	return 0;
}

static
int
tcpSocketBind(void * _this)
{
	TR_TcpSocket this = _this;
	int          bind_ret;

	TR_PARENTRETCALL(TR_TcpSocket, _this, TR_Socket, bind, bind_ret);

	if (bind_ret != 0) {
		return -1;
	}

	if (listen(TR_socketHandle((TR_Socket)this), SOMAXCONN) != 0) {
		// error
		return -1;
	}
	((TR_Socket)this)->fd_getter = TRUE;

	return 0;
}

static
int
tcpSocketConnect(void * _this)
{
	TR_Socket this = _this;
	int       retval = connect(
			this->handle,
			(struct sockaddr *)&(this->addr),
			this->addrlen);

	((TR_TcpSocket)_this)->connected = TRUE;
	return retval;
}

static
TR_RemoteData
tcpSocketRecv(void * _this, size_t size)
{
	TR_Socket this = _this;
	TR_RemoteData rdata = NULL;
	ssize_t       received;

	size = size>TR_TCP_MAX_READ_BLOCK ? TR_TCP_MAX_READ_BLOCK : size;

	unsigned char buffer[size];

	received = recv(this->handle, buffer, size, this->flags);

	if (-1 == received) {
		rdata = NULL;
	} else {
		rdata = TR_new(TR_RemoteData, buffer, received, this);
	}

	return rdata;
}

static
ssize_t
tcpSocketSend(void * _this, TR_RemoteData data)
{
	TR_Socket this = _this;
	return send(
			this->handle,
			((TR_SizedData)data)->data,
			((TR_SizedData)data)->size,
			this->flags);
}

TR_INIT_IFACE(TR_Class, tcpSocketCtor, NULL, NULL);
TR_INIT_IFACE(
		TR_Socket,
		tcpSocketBind,
		tcpSocketConnect,
		tcpSocketRecv,
		tcpSocketSend);
TR_CREATE_CLASS(
		TR_TcpSocket,
		TR_Socket,
		NULL,
		TR_IF(TR_Class),
		TR_IF(TR_Socket));

// vim: set ts=4 sw=4:
