/**
 * \file
 *
 * \author	Georg Hopp
 *
 * \copyright
 * Copyright © 2014 Georg Hopp
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include "tr/socket.h"
#include "tr/interface/socket.h"
#include "tr/logger.h"
#include "trbase.h"

static
int
udpSocketCtor(void * _this, va_list * params)
{
	TR_Socket this = _this;

	TR_PARENTCALL(TR_UdpSocket, _this, TR_Class, ctor, params);

	this->type = SOCK_DGRAM;

	return 0;
}

static
TR_RemoteData
udpSocketRecv(void * _this, size_t size)
{
	TR_Socket this = _this;
	ssize_t       received;
	TR_RemoteData rdata;
	TR_Socket     remote = TR_new(TR_UdpSocket, this->log, NULL, 0, 0);

	size = size>TR_UDP_MAX_READ_BLOCK ? TR_UDP_MAX_READ_BLOCK : size;

	unsigned char buffer[size];

	remote->addrlen = this->addrlen;

	received = recvfrom(
			this->handle,
			buffer,
			size,
			this->flags,
			&(remote->addr.info),
			&remote->addrlen);

	if (-1 == received) {
		TR_delete(remote);
		rdata = NULL;
	} else {
		rdata = TR_new(TR_RemoteData, buffer, received, remote);
	}

	return rdata;
}

static
ssize_t
udpSocketSend(void * _this, TR_RemoteData data)
{
	TR_Socket this = _this;
	ssize_t send;

	send = sendto(
			this->handle,
			((TR_SizedData)data)->data,
			((TR_SizedData)data)->size,
			this->flags,
			&(data->remote->addr.info),
			data->remote->addrlen);

	if (-1 == send) {
		perror("sendto");
	}

	return send;
}

TR_INIT_IFACE(TR_Class, udpSocketCtor, NULL, NULL);
TR_INIT_IFACE(TR_Socket, NULL, NULL, udpSocketRecv, udpSocketSend);
TR_CREATE_CLASS(
		TR_UdpSocket,
		TR_Socket,
		NULL,
		TR_IF(TR_Class),
		TR_IF(TR_Socket));

// vim: set ts=4 sw=4:
